function v = inverse_times_vector(u,M,N)
% task : This function is needed by 'eigs'. It computes
%        the inverse of AK, the matrix with Kronecker
%        products, minus a real shift times the vector u.
%        This is achieved by solving the Sylvester equation
%        MX+XN=U, where U = 2*[U1 0; 0 -U2] and u(1:n^2)=vec(U1),
%        u(n^2+1,2*n^2)=vec(U2). Upper left block of X
%        contains the first half of v, lower right block
%        contains the second half of v.

[nr,nc] = size(M);

n = nr/2;
zerosn = zeros(n);

U1 = reshape(u(1:n*n,1),n,n);
U2 = reshape(u(n*n+1:2*n*n,1),n,n);
        
U = 2*[U1 zerosn; zerosn -U2];

% MInput, NInput, UInput are the input arguments
% for the Sylvester equation solver.
if ~isreal(M) | ~isreal(N) | ~isreal(U)
    % if any of the matrices, M,N,U is complex
    % then we have a complex Sylvester equation.
    % We can convert the complex Sylvester equation
    % to a real one by doubling the size of the
    % system.
    MInput = [real(M) -imag(M); imag(M) real(M)];
    NInput = [real(N) -imag(N); imag(N) real(N)];
    UInput = [real(U) -imag(U); imag(U) real(U)];
else
    % a real Sylvester equation, no modification to
    % the inputs needed.
    MInput = M;
    NInput = N;
    UInput = U;
end
    
XOutput = mexsyl(MInput,NInput,UInput);

if ~isreal(M) | ~isreal(N) | ~isreal(U)
    % solution of the complex Sylvester
    % equation needs to be transformed back.
    X = XOutput(1:nr,1:nr) + i*XOutput(nr+1:2*nr,1:nr);
else
    X = XOutput;
end

v = [reshape(X(1:n,1:n),n*n,1); reshape(X(n+1:2*n,n+1:2*n),n*n,1)];
return;
