function [gamma, theta] = ddi(A, prtlevel, keyb)
% Emre Mengi and Michael Overton (Last updated on March 16 2005)
%
% call: function [gamma, thetabest] = ddi(A, prtlevel, keyb)
% task
%  Quadratically convergent method to compute the distance
%  to instability of a discrete system.
%
%
% DEFINITIONS
% Discrete distance to instability of a matrix is the spectral norm
% of the smallest perturbation to the matrix such that the resulting
% matrix has an eigenvalue on or outside the unit circle in the
% complex plane, i.e.
%
%       (1)  gamma(A) = min{||E|| : max(|lambda(A+E)|) = 1}
%
%  where lambda(B) is the spectrum (set of eigenvalues) of B and |lambda(B)| 
%  is the vector containing the modulus of the eigenvalues of B. This is equivalent 
%  to the minimization problem
%
%       (2) gamma(A) = min_{theta in [0,2pi)} sigma_{min}(A-e^{i*theta}*I) 
%
%  where sigma_{min} denotes the minimum singular value.
%
%  DETAILS OF THE METHOD
%  The method here minimizes g(theta)=sigma_{min}(A-e^{i*theta}*I). At each
%  iteration there are two steps similar to the ones introduced in [1] to
%  compute the H_{inf} norm. At each iteration given a current estimate of
%  gamma(A), alpha, and the midpoints of the pairs of consecutive angles
%  satisfying alpha = g(theta), thetamid, we first calculate the function
%  g(theta) at the midpoints. The minimum of the function values replaces 
%  the current estimate of gamma(A). Next the angles satisfying alpha = g(theta) 
%  are found by the help of the theorem below. Let the sorted vector 
%  of angles satisfying alpha = g(theta) be theta'. From the vector theta'
%  the new set of midpoints are computed.
%
%  The following theorem which is a slightly modified version of theorem 4
%  in [2] is used to find theta values satisfying alpha = g(theta).
%  Theorem : The matrix pencil F(alpha)-lambda*G(alpha) has the generalized
%            eigenvalue e^{i*theta} if and only if A - e^{i*theta}*I has
%            alpha as one of its singular values where F=[-alpha*I A; I 0]
%            and G=[0 I; A' -alpha*I].
%
%
%  References:
%  1- S.Boyd and V.Balakrishnan. A regularity result for the singular
%     values of a transfer matrix and a quadratically convergent 
%     algorithm for computing its L_{inf}-norm. Systems and Control
%     Letters, 15:1-7, 1990.
%  2- R.Byers. A bisection method for computing the distance of a 
%     stable matrix to the unstable matrices. SIAM Journal on
%     Scientific and Statistical Computing, 9:875-881, 1988.
%
%
%
% Input:
%       A         : Input matrix(must be square)
%       prtlevel  : 1: Print the details at each iteration
%                   0: Do not print(Default).
%       keyb      : 1: Interact with the user
%                   0: Do not interact(Default).
%
% Output:
%       gamma     : Discrete distance to instability of the input matrix A.
%       theta     : The angle where gamma(A) is attained, i.e.
%                   gamma(A) = min_{theta} g(theta).

A = full(A);

if nargin <= 2
    keyb = 0; % default: don't interact with the user
end

if nargin <= 1
    prtlevel = 0; % default: don't print
end




% error checks
if isnaninf(A)|isnaninf([prtlevel keyb])
    error('ddi: nan or inf arguments not allowed')
end

n = length(A);
if size(A) ~= [n,n]
    error('ddi: A must be square')
end




% check whether A is stable or not. If it is not stable return 0
if max(abs(eig(A))) >= 1
    gamma = 0;
    theta = 0;
    return;
end


% initializations
thetamid = [0];
% arbitrary values for the first iteration
alphaold = 0;
alpha = -.1;
thetabestt = [];
thetabest = thetamid;


fail_horizontal = 0;

iter = 0;
radtol = 10^-6*norm(A);


% main loop
while alpha < alphaold & ~fail_horizontal
    iter = iter + 1;


    % this is a quadratically convergent method, the number of
    % iterations is supposed to be at most 6,7.
    if (iter > 20)
        error('too many iterations')
    end

    thetabestt = thetabest;
    alphaold = alpha;


    % compute the function value at each midangle, return
    % the minimum of these function values and the midangle
    % at which the minimum function value is attained
    [alpha,thetabest] = ddi_alpha(A,thetamid);
    
    
    if prtlevel > 0
        fprintf('ddi: thetabest = %22.15e  \n', thetabest);
    end
    
    
    % at the first iteration alpha values are arbitrary
    % update them so that the program enters to the if statement below
    if (iter == 1)
        alphaold = alpha + .1;
    end

	


	

    % if we have a better estimate compared to the previous iteration
    if (alpha < alphaold)
        thetamidold = thetamid;

        % find the zeros of the function h(theta) = g(theta)-alpha
        % return the midangles of the zeros
        thetamid = ddi_theta(A,alpha,thetabest,radtol,prtlevel);

        if keyb ~= 0
            keyboard
        end
        
        % if thetamid returned by ddi_theta is empty, we
        % reached machine precision, current alpha and thetabest
        % are the optimum point and the location where the
        % optimum value is attained.
        fail_horizontal = isempty(thetamid);

    else
    % Otherwise the estimate in the previous iteration is better,
    % so we should have reached machine precision in the previous
    % iteration. Restore the values from the previous iteration.
        thetabest = thetabestt;
        alpha = alphaold;
    end
    
    
    if prtlevel > 0
        fprintf('ddi: iteration %d  \n', iter-1);
        fprintf('ddi: alpha = %22.15e  \n', alpha);
        fprintf('ddi: thetamid = %22.15e  \n', thetamid);
    end
            
end % end of while





if isempty(thetabest)
    error('Failed in the first iteration')
end




% set the return variables, thetabest is already set
gamma = alpha;
theta = thetabest;

return;