function [beta, w] = poly_cdi(P, gamma, prtlevel, keyb)
% Michael Overton and Emre Mengi (Last updated on July 1st 2006)
%
% call: function [beta, w] = poly_cdi(A, gamma. prtlevel, keyb)
%
%  DEFINITION:
%  The continuous distance to instability of the matrix polynomial P 
%  is the spectral norm of the smallest perturbation such that the 
%  resulting matrix has an eigenvalue with non-negative real part, i.e.
%    (1) beta(P,gamma) = inf{||DK_0 DK_1 ... DK_k|| : det(P+DP(z))=0 for 
%                       some z with Re(z) >= 0}, and 
%                       DP(lambda) = sum_j=0^k lambda^j DK_j
%  An equivalent definition in terms of singular values is
%    (2) beta(P,gamma) = min_{w in R} sigma_min P(wi)/p_gamma(|w|)
%  where sigma_min denotes minimum singular value and
%           p_gamma(x) = sqrt(sum_j=0^k gamma_j^2 lambda^2j)
%
%  METHOD:
%  An extension of the quadratically convergent Boyd-Balakrishnan
%  method [1] based on definition (2) is implemented. To minimize the 
%  function f(w)=sigma_min P(wi)/p_gamma(|w|) at each iteration given
%  an estimate alpha and a set containing the midpoints of the intervals
%  in which the inequality f(w) <= alpha is satisfied, the function 
%  f(w) is computed at each midpoint. The estimate is updated
%  to the minimum of these values. Finally all w satisfying alpha=f(w)
%  are found. Using these w values the set of midpoints are refined.
%
%  To find w satisfying alpha = f(w) a *-even polynomial eigenvalue
%  problem is solved (see [2] for details).
%
%  References:
%  1- S.Boyd and V.Balakrishnan. A regularity result for the singular
%     values of a transfer matrix and a quadratically convergent 
%     algorithm for computing its L_{inf}-norm. Systems and Control
%     Letters, 15:1-7, 1990.
%  2- E. Mengi. Measures for Robust Stability and Controllability.
%     Ph.D. Thesis. Courant Institute of Mathematical Sciences.
%
%
%
% Input:
%       P         : Input matrix polynomial(nxnxk)
%       gamma     : Vector of nonnegative scaling factors of size k. 
%       prtlevel  : 1: Print the details at each iteration
%                   0: Do not print(Default).
%       keyb      : 1: Interact with the user
%                   0: Do not interact(Default).
%
% Output:
%       beta      : Distance to instability for the input matrix polynomial P.
%       w         : The point where beta(P) is attained, i.e.
%                   beta(P) = min_{w} f(w).



if nargin <= 3
    keyb = 0; % default: don't interact with the user
end

if nargin <= 2
    prtlevel = 0; % default: don't print
end


% error checks
if nargin < 1
    error('poly_cdi: the input polynomial must be provided')
end

[nr,nc,kp] = size(P);
if nr ~= nc
   error('poly_cdi: P must be square')
end
n = nr;
k = kp-1;



% gamma if provided by user must consist of nonnegative real entries
% not all zero
if nargin >= 2
    [gr,gc] = size(gamma);
    
    if gr ~= 1 | gc ~= kp
        error('poly_cdi: gamma must be row vector of size k')
    end
            
    if sum(gamma-abs(gamma)) ~= 0
        error('poly_cdi: gamma must consist of nonnegative real numbers')
    end
    
    if gamma*gamma' == 0
        error('poly_cdi: gamma must be nonzero')
    end
else
    gamma = ones(1,k+1);
end
    


% error checks
if poly_isnaninf(P) | poly_isnaninf([prtlevel keyb]) | ...
        poly_isnaninf(gamma)
   error('poly_cdi: nan or inf arguments not allowed')
end




% check whether P is stable or not. If it is not stable return 0
if max(real(mypolyeig(P))) >= 0
    beta = 0;
    w = 0;   
    return;
end


% initializations
wmid = [0];
% arbitrary values for the first iteration
alphaold = 0;
alpha = -.1;
wbestt = [];
wbest = wmid;


fail_horizontal = 0;
iter = 0;



maxnorm = 0;
for j = 0:k
    maxnorm = max(maxnorm,norm(P(:,:,j+1)));
end
realtol = 10^-8*maxnorm;


% main loop
while alpha < alphaold & ~fail_horizontal
    iter = iter + 1;

    % this is a quadratically convergent method, the number of
    % iterations is supposed to be at most 6,7.
    if (iter > 20)
        error('poly_cdi: too many iterations')
    end

    wbestt = wbest;
    alphaold = alpha;
    % compute the function value at each midpoint, return
    % the minimum of these function values and the midpoint
    % at which the minimum function value is attained
    [alpha,wbest] = poly_cdi_alpha(P,gamma,wmid);
    
    if prtlevel > 0
        fprintf('poly_cdi: wbest = %22.15e  \n', wbest);
    end
        
    

    % At the first iteration alpha values are arbitrary
    % update them so that the program enters to the if statement below.
    % Also in case the minimum is attained at infinity, reduce the 
    % estimate to the value of function at infinity if it is less than 
    % the first estimate    
    if (iter == 1)
        if gamma(k+1) > 0
            s = min(svd(P(:,:,k+1)))/gamma(k+1);
        end
        if gamma(k+1) > 0 & s < alpha
            alpha = s;
            wbest = inf;
        end
        
        alphaold = alpha + .1;
    end

	

    % if we have a better estimate compared to the previous iteration
    if (alpha < alphaold)
        wmidold = wmid;

        % find the zeros of the function g(w) = f(w)-alpha
        % return the midpoints of the consecutive pair of zeros
        wmid = poly_cdi_w(P,gamma,alpha,wbest,realtol,prtlevel);



        if keyb ~= 0
            keyboard
        end
        

        % if wmid returned by cdi_w is empty, we
        % reached machine accuracy, current alpha and wbest
        % are the optimum values and the location where the
        % optimum value is attained.
        fail_horizontal = isempty(wmid);

        
    else
    % Otherwise the estimate in the previous iteration is better,
    % so we should have reached machine accuracy in the previous
    % iteration. Restore the values from the previous iteration.
        wbest = wbestt;
        alpha = alphaold;
    end
    
    if prtlevel > 0
        fprintf('poly_cdi: iteration %d  \n', iter-1);
        fprintf('poly_cdi: alpha = %22.15e  \n', alpha);
        fprintf('poly_cdi: wmid = %22.15e  \n', wmid);    
    end
    
end % end of main loop, while alpha < alphaold ...



if isempty(wbest)

    error('poly_cdi: Failed in the first iteration')
end



% set the return variables
beta = alpha;
w = wbest;

return;