function thetamid = poly_ddi_theta(P, gamma, alpha, thetawant, radtol,...
                                        smalltol,prtlevel)
% Emre Mengi and Michael Overton (Last updated on June 30 2006)
%
% called by poly_ddi.m
%
% Given an estimate alpha for the discrete dista`nce to instability find
% theta values satisfying
%
%       (*)     alpha=g(theta) 
%
% where g(theta)=svd_{min}(P(e^(i*theta)))/||gamma||. Let the vector 
% theta'=[theta(1),theta(2),..theta(m)] contain the values satisfying (*) in
% increasing order, then the vector thetamid filled with the midpoints of 
% the consecutive pairs in theta' is returned. Formally 
% thetamid = [thetamid(1),thetamid(2),...,thetamid(m)] where 
%
%         thetamid(i) = (theta(i) + theta(i+1)) / 2           i < m 
%                       (theta(1) + theta(m) + 2*pi) / 2      i = m
%
% Input:
%       P           : Input matrix polynomial (must be square).
%       gamma       : Vector os scaling parameters.
%       alpha       : Current estimate for the discrete distance to instability.
%       thetawant   : The location where the previous estimate value is
%                     attained.
%       radtol      : A tolerance value to accept whether the modulus of a
%                     generalized eigenvalue of the pencil F(alpha)-lambda*G(alpha)
%                     is one or not.(See the comments at the beginning of 
%                     ddi.m for the definition of F(alpha) and
%                     G(alpha).
%       prtlevel    : 1: print the details.
%                     0: do not print.
%
% Output:
%       thetamid    : Vector of midangles, i.e. thetamid(i) = (theta(i) + theta(i+1)) / 2
%                     for 1 <= i <= m-1 and thetamid(m) = (theta(1) + theta(m) + 2*pi) / 2 
%                     where theta(1),theta(2),...,theta(m) are theta values
%                     satisfying alpha=g(theta).



% get the size of the input matrix and check whether it's square or not.
[nr,nc,kp] = size(P);
if nr ~= nc
    error('P must be square')
end
n = nr;
k = kp-1;


% To compute the intersection points, solve a *-palindromic polynomial
In = eye(n);
Zn = zeros(n);
gamma_norm = norm(gamma);

for j = 0:2*k
    if mod(j,2)
        if j == k % j is odd and equal to k
            Q(:,:,j+1) = -alpha*gamma_norm*eye(2*n);
        else    % j is odd not equal to k
            Q(:,:,j+1) = zeros(2*n);
        end
    else
        if j == k % j is even and equal to k
            Q(:,:,j+1) = [-alpha*gamma_norm*In P(:,:,j/2+1); ...
                          P(:,:,j/2+1)' -alpha*gamma_norm*In];
        else % if j is even and not equal to k
            Q(:,:,j+1) = [Zn P(:,:,j/2+1); ...
                          P(:,:,k-j/2+1)' Zn];
        end
    end
end


eQ = mypolyeig(Q);



% extract the eigenvalues whose moduli are close enough to 1(by the
% moduli close enough to 1 we mean the moduli which are apart from 
% 1 at most by the tolerance value).
ind = find(abs(abs(eQ)-1) < radtol);
eQ = eQ(ind);



if (isempty(eQ)) % check if eFG contains an eigenvalue with modulus 1.
    thetamid = [];
else
    % sort eFG according to the angle of the components
    theta = 2*angle(eQ);
    
    % push the angles into the interval [-pi,pi] 
    for j = 1:length(theta)
       if theta(j) > pi
           theta(j) = theta(j) - 2*pi;
       end
       
       if theta(j) < -pi
           theta(j) = theta(j) + 2*pi;
       end
    end
    % sort the angles
    theta = sort(theta);


    ind2 = [];
    % for each j check if alpha is the smallest singular value of P(e^{i*theta})/||gamma||
    % previous step guaranteed that alpha is a singular value of P(e^{i*theta})/||gamma||
    for j = 1:length(theta)
        if (theta(j) < 0)
            theta(j) = theta(j)+2*pi;
        end
        
        s = compute_sing(P,gamma,exp(i*theta(j)));   
        
        if abs(s-alpha) < smalltol
            ind2 = [ind2; j];
        end
    end


    removed = length(theta) - length(ind2);

    % remove the theta values for which the matrix A-e^{i*theta}*I has the singular value
    % alpha but not the smallest one.
    if removed > 0
        if prtlevel > 0
            fprintf('poly_ddi_theta: singular value test removed %d eigenvalues \n', removed);
        end

        theta = theta(ind2);
    end




    % all of the eigenvalues shouldn't be removed by the singular value
    % test.
    if (isempty(theta))
        warning('singular value test removed all of the intersection points')
        thetamid = [];
        
        return;
    end
    
    
    
    theta = sort(theta);
    
    % organize in pairs and take midpoints
    ind = 0;
    
    
    % shift thetawant, the angle from the previous iteration, into
    % the interval [0,2pi]
    if (thetawant < 0)
        thetawant = thetawant + 2*pi;
    end


    for j=1:length(theta)
        thetalow = theta(j);


        if (j < length(theta))
            thetahigh = theta(j+1);
        else
            % the last interval wraps around
            thetahigh = theta(1)+2*pi;
        end



        % before taking the midpoint, if this interval is not very short,
        % check and see if thetawant is in this interval, well away from the
        % end points.  If so, break this pair into two pairs, one above
        % and one below thetawant, and take midpoints of both.
        inttol = .01 * (thetahigh - thetalow);

        if (thetawant+2*pi > thetalow + inttol & ...
                thetawant+2*pi < thetahigh - inttol)
            thetawantt = thetawant + 2*pi;
        else
            thetawantt = thetawant;
        end


        if thetawantt > thetalow + inttol & ...
                thetawantt < thetahigh - inttol

            % lower midpoint
            ind = ind + 1;
            thetamid(ind,1) = (thetalow + thetawantt)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end		
	
		
            % upper midpoint
            ind = ind + 1;
            thetamid(ind,1) = (thetawantt + thetahigh)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
        else


            % otherwise, if thetawant is not in the interval
            % take the midpoint of thetalow and thetahigh
            ind = ind + 1;
            thetamid(ind,1) = (thetalow + thetahigh)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
        end % end of else
        
    end % end of for

	

end % end of else (isempty(eQ))

return;