function [ynew,newimagtol] = poly_pspa_imag(P, epsln, gamma, x, ywant, iter, imagtol, ...
     plotfig, prtlevel, eigsolver)
% Emre Mengi and Michael Overton
% (Last updated on June 30 2006)
%
% called by poly_pspa.m
%
% Search for intersections between vertical line with given x component
% and the polynomial pseudospectrum.  Start by looking for imaginary eigenvalues 
% of *-even matrix polynomial; if there are none, return the empty vector for ynew.
% Otherwise, remove any non-extreme imaginary eigenvalue that doesn't correspond 
% to the smallest singular value.  Then sort the eigenvalues into pairs.
% As do so, ensure that "ywant" (a scalar) is in the eigenvalue list, and if 
% not, add an extra pair above and below ywant (unless the pair containing 
% ywant is already a very short interval, indicating near convergence to a
% maximizer).  The omission must be caused by rounding, and if we overlook 
% this, the process could terminate to a local minimizer of the 
% pseudospectrum instead of a global maximizer. Finally, return the midpoints; 
% there must be at least one.

Preal = isreal(P);
[nr,nc,kp] = size(P);
n = nr;
k = kp-1;
In = eye(n);
Zn = zeros(n);

smalltol = imagtol;

% create the *-even polynomial
for l = 0:2*k
    
    
    if l <= k
        Bl = Zn;
        for j = l:k
            Bl = Bl + comb(j,l)*x^(j-l)*P(:,:,j+1);
        end
    end
    
    if mod(l,2) == 0
        bl = 0;
        for j = l/2:k
            bl = bl + comb(j,l/2)*gamma(j+1)^2*x^(2*j-l);
        end
        bl = bl*(-1)^(l/2+1);
    end
    
    if mod(l,2)
        if l <= k
            Q(:,:,l+1) = [Zn -Bl'; Bl Zn];
        else
            Q(:,:,l+1) = zeros(2*n);
        end
    else
        if l == 0
            Q(:,:,l+1) = [bl*epsln*In Bl'; Bl -epsln*In];
        else
            if l <= k
                Q(:,:,l+1) = [bl*epsln*In Bl'; Bl Zn];
            else
                Q(:,:,l+1) = [bl*epsln*In Zn; Zn Zn];
            end
        end
    end
end
        
              
% compute eigenvalues of the *-even polynomial
if eigsolver == 0        
    % use Matlab
    eQ = mypolyeig(Q);
else
    % this part is yet to be implemented (for now just set eigsolver=0)
    error('the symmetry preserving eigenvalue solvers are yet to be implemented')
end

% keyboard


minreal = min(abs(real(eQ)));

% tolerance is adaptive, i.e. if vertical search fails
% in the first iteration that means tolerance is too small.
% So increase the tolerance.
if (iter == 1) & (minreal > imagtol)
    imagtol = minreal + imagtol;
end

newimagtol = imagtol;


if minreal > imagtol    % check if Q has an imaginary eigenvalue
    ynew = [];
else
    indx = find(abs(real(eQ)) <= imagtol);  % extract such eigenvalues
    % make sure the imaginary parts correspond to the minimum singular 
    % value, not some other singular value
    y = sort(imag(eQ(indx)));  % order them by increasing imaginary part


    % singular value tests
    % remove the imaginary eigenvalues if they don't correspond to the
    % minimum singular value
    indx2 = 1;       % check out the non-extreme imaginary parts
    for j = 2: length(indx)-1   % does nothing if there are only 2
        s = compute_sing(P,gamma,x+i*y(j));
         
        if abs(s-epsln) <= smalltol
            indx2 = [indx2; j];   % accept this eigenvalue
        end
    end   
    indx2 = [indx2; length(indx)];  % last one is an extreme one
    removed = length(indx) - length(indx2);
   
    if removed > 0
        if (prtlevel > 0)
            fprintf('poly_pspa_imag: singular value test removed %d eigenvalues \n', removed)
        end
        y = y(indx2);
    end
    
    
   
  
   npairs = length(y)/2;
   if floor(npairs) ~= npairs
      % odd number of intersection points detected by the vertical search
      error('poly_pspa_imag: odd number of intersection points are detected by the vertical search')
   end
   
   if (npairs == 0)       
       % singular value test removed all of the vertical intersection points
       warning('poly_pspa_imag: singular value test removed all of the intersection points')
   end
   
      
   % organize in pairs and take midpoints
   ind = 0;
   for j=1:length(y)-1        % already checked length(y) is even
      ylow = y(j);
      yhigh = y(j+1);
      % before taking the midpoint, if this interval is not very short,
      % check and see if ywant is in this interval, well away from the
      % end points.  If so, break this pair into two pairs, one above
      % and one below ywant, and take midpoints of both.
      inttol = .01 * (yhigh - ylow);
      if ywant > ylow + inttol & ywant < yhigh - inttol
         ind = ind + 1;
         ynew(ind,1) = (ylow + ywant)/2;
         ind = ind + 1;
         ynew(ind,1) = (ywant + yhigh)/2;
      else
         ind = ind + 1;
         ynew(ind,1) = (ylow + yhigh)/2;
      end
   end
   
   if plotfig > 0
      figure(plotfig)
      plot([x x], [max(y) min(y)], 'r-')       % plot vertical line
      plot(x*ones(length(y)), y, 'g+')          % plot intersection points   
      plot(x*ones(length(ynew)), ynew, 'bx')   % plot midpoints
   end
   
   
   % if A is real, discard the midpoints in the lower half plane
   if Preal
      indx = find(ynew >= 0);
      ynew = ynew(indx);
   end

end % end of else

return;