function [xbest, ybest] = poly_pspa_real(P, epsln, gamma, y, imagtol, plotfig, xold, ...
                                        eigsolver)
% Emre Mengi and Michael Overton
% (Last updated on June 30 2006)
%
% called by poly_pspa.m
%
% For each component in y, search for rightmost intersection between
% horizontal line with given y component and the pseudospectrum.
% The rightmost intersection point is found by solving a *-even
% polynomial eigenvalue problem. Input parameter xold is used only 
% for plotting, if plotfig > 0

[n,nc,kp] = size(P);
k = kp-1;    
In = eye(n);
Zn = zeros(n);

for j=1:length(y)
    % create the *-even polynomial
    for l = 0:2*k
    
    
        if l <= k
            Cl = Zn;
            for m = l:k
                Cl = Cl + comb(m,l)*(-y(j))^(m-l)*(-i)^m*P(:,:,m+1);
            end
        end
    
        if mod(l,2) == 0
            cl = 0;
            for m = l/2:k
                cl = cl + comb(m,l/2)*gamma(m+1)^2*(-y(j))^(2*m-l);
            end
            cl = cl*(-1)^(l/2+1);
        end
    
        if mod(l,2)
            if l <= k
                Q(:,:,l+1) = [Zn -Cl'; Cl Zn];
            else
                Q(:,:,l+1) = zeros(2*n);
            end
        else
            if l == 0
                Q(:,:,l+1) = [cl*epsln*In Cl'; Cl -epsln*In];
            else
                if l <= k
                    Q(:,:,l+1) = [cl*epsln*In Cl'; Cl Zn];
                else
                    Q(:,:,l+1) = [cl*epsln*In Zn; Zn Zn];
                end
            end
        end
    end
    
    
   % compute eigenvalues of the *-even polynomial
   if eigsolver == 0       
       % use built-in eig of matlab to compute Hamiltonian eigenvalues
       eQ = mypolyeig(Q);
   else
       % the eigenvalue solver that preserves the eigenvalue symmetry has
       % yet to be implemented. For now use please 'eig'.
       error('the eigenvalue-solver respecting the symmetry of the spectrum has yet to be implemented')
   end
       
       

   if min(abs(real(eQ))) <= imagtol % check if Q has an imaginary eigenvalue
      indx = find(abs(real(eQ)) <= imagtol);  % extract such eigenvalues
      xnew(j) = max(imag(eQ(indx)));
      
      if plotfig > 0
          figure(plotfig)
          plot([xold xnew(j)], [y(j) y(j)], 'm-')  % plot horizontal line
          plot(xnew(j), y(j), 'b+')       % plot right end point
      
          if isreal(A)
              plot([xold xnew(j)], -[y(j) y(j)], 'm-')  % plot horizontal line
              plot(xnew(j), -y(j), 'b+')    % plot right end point
          end
      end
      
      
   else
       % Normally horizontal search should return an intersection point
       % for each midpoint, however because of rounding errors it failed
       % to do so.       
       xnew(j) = -inf;
   end % end of else
   
end % end of for

if ~isfinite(max(xnew))
    % The horizontal searches failed.
    error('poly_pspa_real: horizontal search failed to return any intersection point')
end

[xbest,ind] = max(xnew);     % furthest to right, returns 1 index
ybest = y(ind);              % corresponding y value

% mark the rightmost intersection point
if plotfig > 0 & xbest > xold
   plot(xbest, ybest, 'b*')
end

return;